#include <flash/flash.h>
#include "lcd.h"
#include "uart.h"
#include "termio.h"
#include "keys.h"
#include "interrupts.h"
#include "elfrun.h"

#define gethex(n)	gethex_ex(n, (GETC)term_getc)

static const char * const flash_error[] = {NULL, "Timeout!", "Odd address!"};

static unsigned char term_getc() {
	unsigned char c = uart_getc();
	uart_putc(c);
	return c;
}

static char getcx() {
	short c;
	for (;;) {
		c = uart_peekc();
		if (keys_peek() == HK_B4)
			return 'x';
		if (c >= 0) {
			uart_putc(c);
			return c;
		}
	}
}

static void writeflash(unsigned long addr, unsigned char len) {
	static unsigned short buf[129];
	unsigned short *t;
	unsigned char i;
	char e;

	if (len & 1) buf[len++] = 0;

	for (t = buf, i = 0; i < len; i += 2)
		*(t++) = gethex(4);

	if ((e = flash_write(addr, buf, len >> 1))) {
		uart_puts(flash_error[(unsigned char)e]);
		return;
	}
}

static void writememory(unsigned long addr, unsigned char len) {
	unsigned char *b = (unsigned char *)addr;
	extern unsigned char used_ram_end;

	if (b < &used_ram_end) {
		uart_puts("Zabranjeno pisanje po BIOS memoriji!");
		return;
	}

	for (; len--; ++b)
		*b = gethex(2);
}

static void _start_elf(void *data) {
	const Elf *be = elf_detect(data);

	if (!be) {
		uart_puts("Nije ELF!");
		return;
	}

	elf_run(be);
}

void flasher() {
	char c;
	unsigned long t, llines = 0, nlines = 0;
	IRQ_HANDLER old_keys;

	old_keys = irq_register(IRQID_KEYS, NULL);

	LCD_puts("Flasher started!");

	for (;;) {
		switch (getcx()) {
			case 'r':
				c = term_getc();
				t = gethex(8);
				switch (c) {
					case 'b':
						uart_puthex(*(unsigned char*)t);
						break;
					case 'w':
						uart_puthex(*(unsigned short*)t);
						break;
					case 'l':
						uart_puthex(*(unsigned long*)t);
						break;
					default:
						goto err;
				}
				break;
			case 'w':
				c = term_getc();
				t = gethex(8);
				switch (c) {
					case 'b':
						*(unsigned char*)t = gethex(2);
						break;
					case 'w':
						*(unsigned short*)t = gethex(4);
						break;
					case 'l':
						*(unsigned long*)t = gethex(8);
						break;
					default:
						goto err;
				}
				break;
			case 'm':
				t = gethex(8);
				writememory(t, gethex(2));
				break;
			case 'x':
				goto kraj;
			case 'c':
				((void (*)())gethex(8))();
				break;
			case 'C':
				_start_elf((void*)gethex(8));
				break;
			case 'i':
				uart_puthex(flash_getid());
				break;
			case 'e':
				flash_erase_sector(gethex(2));
				break;
/*			Ovo nema smisla jer samog sebe unisti... :)
			case 'E':
				flash_erase();
				break;*/
			case 'f':
				t = gethex(8);
				writeflash(t, gethex(2));
				break;

			case 'l':
				nlines = gethex(8);
				llines = 0;
				LCD_puts("Linija:");
				LCD_putdec(nlines);
				LCD_putc('\n');
				break;

			case '\n':
				if (nlines) {
					if (llines < nlines) {
						LCD_putc('\r');
						LCD_putdec(llines++*100/nlines);
						LCD_putc('%');
					} else {
						LCD_puts("\rDone!");
						nlines = 0;
					}
				}
				break;

			default:
err:
				uart_puts("Unknown command!");
			case 'z':
			case 'Z':
			case '\r':
			case '#':
				break;
		}
	}

kraj:
	irq_register(IRQID_KEYS, old_keys);
	uart_puts("Bye!");
}

// za trap0
char trap_flash_erase_sector(unsigned long d1) {
	return flash_erase_sector(d1);
}

char trap_flash_write(unsigned long d1, unsigned long d2, unsigned long d3) {
	return flash_write(d1, (void*)d2, d3);
}

long trap_flash_address_by_id(unsigned long d1) {
	return (long)flash_address_by_id(d1);
}
