#include <cpu.h>
#include "interrupts.h"
#include "lcd.h"

#define CALL_IRQHANDLER(id, sp)	if (irq_table[id]) irq_table[id](sp);

static IRQ_HANDLER irq_table[NUM_IRQS];

void irq_init() {
	IVR = 0x40;
}

IRQ_HANDLER irq_register(unsigned short id, IRQ_HANDLER func) {
	IRQ_HANDLER old;
	old = irq_table[id];
	irq_table[id] = func;
	return old;
}

long trap_irq_register(unsigned long d1, unsigned long d2) {
	return (long)irq_register(d1, (IRQ_HANDLER)d2);
}

void fatal_exception() {
	LCD_puts("Fatal!!!");
}

void divide_by_zero() {
	LCD_puts("Divide by zero!!!");
}

void illegal_instruction() {
	LCD_puts("Illegal instruction!!!");
}

void address_error() {
	LCD_puts("Address error!!!");
}

void bus_error() {
	LCD_puts("BUS error!!!");
}

void irq_level0(void *sp) {
}

// hotsync
void irq_level1(void *sp) {
	CALL_IRQHANDLER(IRQID_KEYS, sp);
	ISR |= INTR_IRQ1;
}

void irq_level2(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ2;
}

void irq_level3(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ3;
}

void irq_level4(void *sp) {
	if (ISR & INTR_WDT) {
		//TODO
	}

	if (ISR & INTR_SAM) {
		CALL_IRQHANDLER(IRQID_SAM, sp);
//		RTCISR = 0xFFFF;
	}

	if (ISR & INTR_UART) {
		CALL_IRQHANDLER(IRQID_UART, sp);
	}

	if (ISR & INTR_SPI) {
		SPIMCONT &= 0x0080;
	}

	if (ISR & INTR_RTC) {
		// TODO
	}

	if (ISR & INTR_KB) {
		// TODO
	}

	if (ISR & (INTR_INT0 | INTR_INT1 | INTR_INT2 | INTR_INT3)) {
		CALL_IRQHANDLER(IRQID_KEYS, sp);
		PDDATA |= 0x0F;
	}
}

void irq_level5(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ5;
}

void irq_level6(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ6 | INTR_TMR | INTR_PWM;
}

void irq_level7(void *sp) {
	// ugasi ih:
	IMR |= INTR_EMIQ;
}
