#include <cpu.h>
#include "keys.h"
#include "interrupts.h"

static volatile unsigned char key;
static IRQ_HANDLER old_keys_irq;

#define DELAY()	{NOP();NOP();NOP();}

unsigned char keys_read() {
	unsigned char pwr, up, down, b1, b2, b3, b4, sync;
	unsigned char oldpdirqeg = PDIRQEG;
//	CPU_STATUS oldstat = GET_STATUS();

//	DI();
	PDIRQEG = 0x00;

	KEYMASK(KEYROW0 | KEYROW1);
	DELAY();
	up = (PDDATA & 0x02) ? HK_UP : 0;
	pwr = (PDDATA & 0x01) ? HK_POWER : 0;

	KEYMASK(KEYROW0);
	DELAY();
	down = (!up && (PDDATA & 0x02)) ? HK_DOWN : 0;

	KEYMASK(KEYROW1);
	DELAY();
	b1 = (!pwr && (PDDATA & 0x01)) ? HK_B1 : 0;
	b2 = (!up && (PDDATA & 0x02)) ? HK_B2 : 0;
	b3 = (PDDATA & 0x04) ? HK_B3 : 0;
	b4 = (PDDATA & 0x08) ? HK_B4 : 0;

	KEYMASK(0);
	DELAY();
	if (!b2 && !up && !down && (PDDATA & 0x02)) {
		up = HK_UP;
		b2 = HK_B2;
	}

	sync = !(PDDATA & 0x10) ? HK_SYNC : 0;

	PDIRQEG = oldpdirqeg;
//	SET_STATUS(oldstat);

	return pwr | sync | up | down | b1 | b2 | b3 | b4;
}

unsigned char keys_peek() {
	return key;
}

unsigned char keys_wait() {
	short i;
	key = 0x00;
ponovo:
	while (!key) HALT();
	for (i = 0; key; ++i) {
		if (key & 0x01)
			return 1 << i;
		key >>= 1;
	}
	goto ponovo;	// make compiler happy... :)
}

static void keys_irq() {
	key = keys_read();
}

void keys_init() {
	// hardkeys
	KEYMASK(0);
	PDPOL = 0x0F;
	PDIRQEG = 0x0F;
	PDKBEN = 0x00;
	PDIRQEN = 0x0F;
	PDDATA |= 0x0F;

	// hotsync
	ICR &= ~0x8000;
	ICR |= 0x0800;
	ISR |= INTR_IRQ1;

	IMR &= ~(INTR_INT0 | INTR_INT1 | INTR_INT2 | INTR_INT3 | INTR_IRQ1);

	old_keys_irq = irq_register(IRQID_KEYS, keys_irq);
}

void keys_deinit() {
	irq_register(IRQID_KEYS, old_keys_irq);
}
