#include <flash/flash.h>
#include <elf/elf.h>
#include "elfrun.h"
#include "lcd.h"
#include "uart.h"
#include "flasher.h"
#include "numpick.h"
#include "ad.h"

#define MAX_APPS	4

static void adj_lcd_numpick_callback(short i) {
	LCD_bias(i << 2);
}

static void adj_lcd_bl_callback(short i) {
	LCD_backlight(i);
}

static void adj_lcd() {
	LCD_puts("Pick contrast value:");
	LCD_bias(32 << 2);
	numpick_ex(32, 24, 63, adj_lcd_numpick_callback);

	LCD_puts("Backlight power:");
	LCD_backlight(0);
	numpick_ex(0, 0, 1, adj_lcd_bl_callback);
}

static void comm_settings() {
	static unsigned short speeds[] = {
		UART_B57600,
		UART_B28800,
		UART_B19200,
		UART_B14400,
		UART_B9600
	};
	short i;

	LCD_puts("Odaberi brzinu:");
	LCD_puts(" 0: 57600\n"
	         " 1: 28800\n"
	         " 2: 19200\n"
	         " 3: 14400\n"
	         " 4: 9600");
	i = numpick(4, 0, 4);
	uart_set_baudrate(speeds[(unsigned short)i]);
}

void bios_main() {
	short i = 0, j, nbes, ns;
	const Elf *bes[MAX_APPS];
	int retval;

	extern char bss_end, bss_start, used_ram_end, rom_start;
	LCD_puts("BSS size:");
	LCD_puthex(&bss_end - &bss_start);
	LCD_puts("RAM free from:");
	LCD_puthex((unsigned long)&used_ram_end);
	LCD_puts("Battery:");
	LCD_putdec(AD_read_battery8());

	LCD_puts("\n\nBIOS started! \1");
ponovi:
	// srcha programe po flashu
	nbes = 0;
	ns = flash_get_num_sectors();
	for (j = 1; j < ns; ++j) {
		bes[nbes] = elf_detect(&rom_start + flash_address_by_id(j));
		if (bes[nbes] && ++nbes > MAX_APPS)
			break;
	}

	LCD_putc('\n');
	LCD_puts("Pick a choice:");
	LCD_puts(" 0: LCD settings");
	LCD_puts(" 1: comm settings");
	LCD_puts(" 2: start flasher");
	for (j = 0; j < nbes; ++j) {
		LCD_putc(' ');
		LCD_putc('3' + j);
		LCD_putc(';');
		LCD_putc(' ');
		LCD_puts(elf_get_appname(bes[j]));
	}

	i = numpick(i, 0, 2 + nbes);

	switch (i) {
		case 0:
			adj_lcd();
			break;
		case 1:
			comm_settings();
			break;
		case 2:
			flasher();
			break;
		default:
			retval = elf_run(bes[i - 3]);
			LCD_puts("Returned:");
			LCD_puthex(retval);
			break;
	}

	LCD_putc('\n');
	LCD_puts("Gotovo!");
	LCD_puts("Resuming BIOS main.");
	goto ponovi;
}
