#include "../include/cpu.h"

static const char errs[] = "Error!";

static char buf[100] = "", *buf_ptr = buf;

static void putc(char c);
static void printhex(unsigned long ul);
static void puts(const char *s);

static char sergetc() {
	while (!(URX & 0x2000));
	return URX;
}

static void readline() {
	char c, *t;
	for (t = buf; (c = sergetc()) != '\n'; ++t)
		*t = c;
	*t = 0;
	puts(buf);
	buf_ptr = buf;
}

static char getc() {
	char c = *buf_ptr;
	if (c) buf_ptr++;
	return c;
}

static void putc(char c) {
	while (UTX & 0x0400);
	UTX = c;
}

static void puts(const char *t) {
	for (; *t; ++t)
		putc(*t);
	putc('\n');
}

static unsigned long gethex(short n) {
	unsigned long r = 0;
	register short i;
	char c;
	for (i = 0; i < n; ++i) {
		r <<= 4;
		c = getc();
		if (c >= 'A' && c <= 'F')
			r |= c - 'A' + 10;
		else if (c >= 'a' && c <= 'f')
			r |= c - 'a' + 10;
		else if (c >= '0' && c <= '9')
			r |= c - '0';
		else {
			puts(errs);
			return 0;
		}
	}
	return r;
}

static void printhex(unsigned long ul) {
	static const char hexchars[] = "0123456789ABCDEF";
	short int i;
	putc('0');
	putc('x');
	for (i = 0; i < 8; ++i, ul<<=4)
		putc(hexchars[(ul>>28) & 0x0F]);
	putc('\n');
}

#define FLASH	((volatile unsigned short *)0x10C00000)
#define FLASH_RESET()	FLASH[0x0000] = 0xF0
#define FLASH_CMD(x)	{ \
				FLASH[0x0555] = 0xAA; \
				FLASH[0x02AA] = 0x55; \
				FLASH[0x0555] = (x); \
			}

static void getinfo() {
	FLASH_RESET();
	FLASH_CMD(0x90);
	puts("Info:");
	printhex(FLASH[0]);
	printhex(FLASH[1]);
	FLASH_RESET();
}

static char waitflashop(unsigned long addr, unsigned char qn) {
	unsigned short l;
	while (((l = FLASH[addr]) ^ FLASH[addr]) & qn) {
		if (l & 0x20) {
			puts("Flash error!");
			return 0;
		}
	}
	return 1;
}

static char writeword(unsigned long addr, unsigned short d) {
	FLASH[addr] = 0xA0;
	FLASH[addr] = d;
	return waitflashop(addr, 0x40);
}

static void writeflash(unsigned long addr, unsigned char n) {
	if (n & 1) {
		puts("Odd number od bytes!");
		return;
	}
	FLASH_RESET();
	FLASH_CMD(0x20);
	for (addr >>= 1; n; n-=2, ++addr) {
		if (!writeword(addr, gethex(4)))
			break;
	}
	FLASH[0] = 0x90;
	FLASH[0] = 0xF0;
}

static void erasesector(unsigned char id) {
	static const unsigned long bsa[] = {0x00000, 0x04000, 0x06000, 0x08000};
	unsigned long saddr;

	if (id > 34) {
		puts(errs);
		return;
	}

	if (id < 4)
		saddr = bsa[id];
	else
		saddr = (id - 3) << 16;

	FLASH_RESET();
	FLASH_CMD(0x80);
	FLASH[0x555] = 0xAA;
	FLASH[0x2AA] = 0x55;
	FLASH[saddr] = 0x30;
	waitflashop(saddr, 0x04);
}

static void chiperase() {
	FLASH_RESET();
	FLASH_CMD(0x80);
	FLASH[0x555] = 0xAA;
	FLASH[0x2AA] = 0x55;
	FLASH[0x555] = 0x10;
	waitflashop(0, 0x04);
}

void _start() {
	unsigned long t;
	char c;

	USTCNT |= 0xE000;
	CSA = 0x01A9;

	puts("\nHello!");

	for (;;) {
		readline();
		switch (getc()) {
			case 'r':
				c = getc();
				t = gethex(8);
				switch (c) {
					case 'b':
						printhex(*(unsigned char*)t);
						break;
					case 'w':
						printhex(*(unsigned short*)t);
						break;
					case 'l':
						printhex(*(unsigned long*)t);
						break;
					default:
						goto err;
				}
				break;
			case 'w':
				c = getc();
				t = gethex(8);
				switch (c) {
					case 'b':
						*(unsigned char*)t = gethex(2);
						break;
					case 'w':
						*(unsigned short*)t = gethex(4);
						break;
					case 'l':
						*(unsigned long*)t = gethex(8);
						break;
					default:
						goto err;
				}
				break;
			case 'x':
				puts("Bye!");
				return;
			case 'c':
				((void (*)())gethex(8))();
				break;
/*			case 't':
				puts("test!");
				test();
				break;*/
			case 'i':
				getinfo();
				break;
			case 'e':
				erasesector(gethex(2));
				break;
			case 'E':
				chiperase();
				break;
			case 'f':
				t = gethex(8);
				writeflash(t, gethex(2));
				break;

			case 'R':
				puts("Reset...");
				RESET();
				break;

			default:
err:
				puts(errs);

			case 'z':
			case 'Z':
			case '\n':
			case '\r':
			case '#':
				break;
		}
	}
}
