#ifndef _CPU_H_
#define _CPU_H_

#ifndef __ASSEMBLY__

# define CPUR08(o)	(*(volatile unsigned char *)(0xFFFFF000 | (o)))
# define CPUR16(o)	(*(volatile unsigned short *)(0xFFFFF000 | (o)))
# define CPUR32(o)	(*(volatile unsigned long *)(0xFFFFF000 | (o)))

# define EI()		asm volatile("and.w #0xF8FF, %sr")
# define DI()		asm volatile("or.w #0x0700, %sr")
# define HALT()		asm volatile("stop #0x2000")
# define NOP()		asm volatile("nop")
# define RESET()	asm volatile("reset")

typedef union {
	struct {
		short T1	: 1;
		short T0	: 1;
		short S		: 1;
		short M		: 1;
		short zero1	: 1;
		short int_mask	: 3;
		short zero2	: 3;
		short X		: 1;
		short N		: 1;
		short Z		: 1;
		short V		: 1;
		short C		: 1;
	} s;
	unsigned short w;
} CPU_STATUS;

static inline CPU_STATUS GET_STATUS() {
	CPU_STATUS stat;
	asm volatile("move.w %%sr, %0" : "=d"(stat.w));
	return stat;
}

static inline void SET_STATUS(CPU_STATUS x) {
	asm volatile("move.w %0, %%sr" : : "d"(x.w));
}

#else /* ifndef __ASSEMBLY__ */

# define CPUR08(o)	0xFFFFF000 + o
# define CPUR16(o)	0xFFFFF000 + o
# define CPUR32(o)	0xFFFFF000 + o

# define EI		and.w #0xF8FF, %sr
# define DI		or.w #0x0700, %sr
# define HALT		stop #0x2000

#endif /* __ASSEMBLY */

#define SCR		CPUR08(0x000)
#define ID		CPUR32(0x004)
#define CSGBA		CPUR16(0x100)
#define CSGBB		CPUR16(0x102)
#define CSGBC		CPUR16(0x104)
#define CSGBD		CPUR16(0x106)
#define CSA		CPUR16(0x110)
#define CSB		CPUR16(0x112)
#define CSC		CPUR16(0x114)
#define CSD		CPUR16(0x116)
#define PLLCR		CPUR16(0x200)
#define PLLFSR		CPUR16(0x202)
#define PCTLR		CPUR08(0x207)
#define IVR		CPUR08(0x300)
#define ICR		CPUR16(0x302)
#define IMR		CPUR32(0x304)
#define ISR		CPUR32(0x30C)
#define IPR		CPUR32(0x310)
#define PADIR		CPUR08(0x400)
#define PADATA		CPUR08(0x401)
#define PAPUEN		CPUR08(0x402)
#define PBDIR		CPUR08(0x408)
#define PBDATA		CPUR08(0x409)
#define PBPUEN		CPUR08(0x40A)
#define PBSEL		CPUR08(0x40B)
#define PCDIR		CPUR08(0x410)
#define PCDATA		CPUR08(0x411)
#define PCPDEN		CPUR08(0x412)
#define PCSEL		CPUR08(0x413)
#define PDDIR		CPUR08(0x418)
#define PDDATA		CPUR08(0x419)
#define PDPUEN		CPUR08(0x41A)
#define PDSEL		CPUR08(0x41B)
#define PDPOL		CPUR08(0x41C)
#define PDIRQEN		CPUR08(0x41D)
#define PDKBEN		CPUR08(0x41E)
#define PDIRQEG		CPUR08(0x41F)
#define PEDIR		CPUR08(0x420)
#define PEDATA		CPUR08(0x421)
#define PEPUEN		CPUR08(0x422)
#define PESEL		CPUR08(0x423)
#define PFDIR		CPUR08(0x428)
#define PFDATA		CPUR08(0x429)
#define PFPUEN		CPUR08(0x42A)
#define PFSEL		CPUR08(0x42B)
#define PGDIR		CPUR08(0x430)
#define PGDATA		CPUR08(0x431)
#define PGPUEN		CPUR08(0x432)
#define PGSEL		CPUR08(0x433)
#define SPIMDATA	CPUR16(0x800)
#define SPIMCONT	CPUR16(0x802)
#define USTCNT		CPUR16(0x900)
#define UBAUD		CPUR16(0x902)
#define URX		CPUR16(0x904)
#define UTX		CPUR16(0x906)
#define UMISC		CPUR16(0x908)
#define NIPR		CPUR16(0x90A)
#define LSSA		CPUR32(0xA00)
#define LVPW		CPUR08(0xA05)
#define LXMAX		CPUR16(0xA08)
#define LYMAX		CPUR16(0xA0A)
#define LCXP		CPUR16(0xA18)
#define LCYP		CPUR16(0xA1A)
#define LCWCH		CPUR16(0xA1C)
#define LBLKC		CPUR08(0xA1F)
#define LPICF		CPUR08(0xA20)
#define LPOLCF		CPUR08(0xA21)
#define LACDRC		CPUR08(0xA23)
#define LPXCD		CPUR08(0xA25)
#define LCKCON		CPUR08(0xA27)
#define LRRA		CPUR08(0xA29)
#define LPOSR		CPUR08(0xA2D)
#define LFRCM		CPUR08(0xA31)
#define LGPMR		CPUR08(0xA33)
#define PWMR		CPUR16(0xA36)
#define WATCHDOG	CPUR16(0xB0A)
#define RTCCTL		CPUR16(0xB0C)
#define RTCISR		CPUR16(0xB0E)
#define RTCIENR		CPUR16(0xB10)
#define DRAMMC		CPUR16(0xC00)
#define DRAMC		CPUR16(0xC02)

#endif /* _CPU_H_ */
