#include <cpu.h>
#include "lcd.h"
#include "string.h"
#include "keys.h"

static const char errs[] = "Error!";

static void putc(char c);
void printhex(unsigned long ul);

static char getc() {
	unsigned short c;
	while (!((c = URX) & 0x2000));
	putc(c);
//	LCD_putc(c);
	return c;
}

static void putc(char c) {
	while (!(UTX & 0x4000));
	UTX = c;
}

void puts(const char *t) {
	for (; *t; ++t)
		putc(*t);
	putc('\n');
}

static unsigned long gethex(short n) {
	unsigned long r = 0;
	register short i;
	char c;
	for (i = 0; i < n; ++i) {
		r <<= 4;
		c = getc();
		if (c >= 'A' && c <= 'F')
			r |= c - 'A' + 10;
		else if (c >= 'a' && c <= 'f')
			r |= c - 'a' + 10;
		else if (c >= '0' && c <= '9')
			r |= c - '0';
		else {
			puts(errs);
			return 0;
		}
	}
	return r;
}

void printhex(unsigned long ul) {
	static const char hexchars[] = "0123456789ABCDEF";
	short int i;
	putc('0');
	putc('x');
	for (i = 0; i < 8; ++i, ul<<=4)
		putc(hexchars[(ul>>28) & 0x0F]);
	putc('\n');
}
/*
static void LCD_puts(const char *s) {
	while (*s)
		LCD_putc(*(s++));
	LCD_putc('\n');
}*/

void trap0(long d0, long d1, long d2, long d3) {
	printhex(d0);
	printhex(d1);
	printhex(d2);
	printhex(d3);
}

static void test2() {
	irq_init();
	keys_init();

	printhex(GET_STATUS().w);
	printhex(IMR);
}

static void test() {
	printhex(PBDATA);
	printhex(keys_wait());
}

void _start() {
	unsigned long t;
	char c;

	extern short exceptions, exceptions_end;
	memcpyw((void*)0, &exceptions, &exceptions_end - &exceptions);

//LCD_init();
//test();
	for (;;) {
		putc('>');
		switch (getc()) {
			case 'r':
				c = getc();
				t = gethex(8);
				putc('\n');
				switch (c) {
					case 'b':
						printhex(*(unsigned char*)t);
						break;
					case 'w':
						printhex(*(unsigned short*)t);
						break;
					case 'l':
						printhex(*(unsigned long*)t);
						break;
					default:
						goto err;
				}
				break;
			case 'w':
				c = getc();
				t = gethex(8);
				switch (c) {
					case 'b':
						*(unsigned char*)t = gethex(2);
						break;
					case 'w':
						*(unsigned short*)t = gethex(4);
						break;
					case 'l':
						*(unsigned long*)t = gethex(8);
						break;
					default:
						goto err;
				}
				break;
			case 'x':
				puts("Bye!");
				return;
			case 't':
				puts("Test!");
				test();
				puts("Done!");
				break;
			case 'T':
				test2();
				break;

			default:
err:
				puts(errs);
			case '\n':
			case '\r':
			case '#':
				break;
		}
		while (getc() != '\n');
	}
}
