#include "../include/cpu.h"
#include "interrupts.h"
#include "lcd.h"

#define CALL_IRQHANDLER(id, sp)	if (irq_table[id]) irq_table[id](sp);

static IRQ_HANDLER irq_table[NUM_IRQS];

void irq_init() {
puts("init");
	IVR = 0x40;
	IMR = 0xFFFFFFFF;
	EI();
}

IRQ_HANDLER irq_register(unsigned short id, IRQ_HANDLER func) {
	IRQ_HANDLER old;
	old = irq_table[id];
	irq_table[id] = func;
puts("reg");
	return old;
}

void fatal_exception() {
	puts("Fatal!!!");
	DI();
	for(;;);
}

void irq_level0(void *sp) {
}

// hotsync
void irq_level1(void *sp) {
	CALL_IRQHANDLER(IRQID_KEYS, sp);
	ISR |= INTR_IRQ1;
}

void irq_level2(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ2;
}

void irq_level3(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ3;
}

void irq_level4(void *sp) {
	if (ISR & INTR_WDT) {
		//TODO
	}

	if (ISR & INTR_UART) {
		//TODO
	}

	if (ISR & INTR_SPI) {
		SPIMCONT &= 0x0080;
	}

	if (ISR & INTR_RTC) {
		// TODO
	}

	if (ISR & INTR_KB) {
		// TODO
	}

	if (ISR & (INTR_INT0 | INTR_INT1 | INTR_INT2 | INTR_INT3)) {
		CALL_IRQHANDLER(IRQID_KEYS, sp);
		PDDATA |= 0x0F;
	}
}

void irq_level5(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ5;
}

void irq_level6(void *sp) {
	// ugasi ih:
	IMR |= INTR_IRQ6 | INTR_TMR | INTR_PWM;
}

void irq_level7(void *sp) {
	// ugasi ih:
	IMR |= INTR_EMIQ;
}
