#include <cpu.h>
#include "keys.h"
#include "interrupts.h"

static volatile unsigned char key;
static IRQ_HANDLER old_keys_irq;

#define DELAY()	{NOP();NOP();NOP(); \
NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();}

unsigned char keys_read() {
	unsigned char pwr, up, down, b1, b2, b3, b4, sync;
	unsigned char pdd1, pdd2, pdd3, pdd4;

	KEYMASK(KEYROW0 | KEYROW1);
	DELAY();
	pdd1 = PDDATA;

	KEYMASK(KEYROW0);
	DELAY();
	pdd2 = PDDATA;

	KEYMASK(KEYROW1);
	DELAY();
	pdd3 = PDDATA;

	KEYMASK(0);
	DELAY();
	pdd4 = PDDATA;

	up = (pdd1 & 0x02) ? HK_UP : 0;
	pwr = (pdd1 & 0x01) ? HK_POWER : 0;

	down = (!up && (pdd2 & 0x02)) ? HK_DOWN : 0;

	b1 = (!pwr && (pdd3 & 0x01)) ? HK_B1 : 0;
	b2 = (!up && (pdd3 & 0x02)) ? HK_B2 : 0;
	b3 = (pdd3 & 0x04) ? HK_B3 : 0;
	b4 = (pdd3 & 0x08) ? HK_B4 : 0;

	if (!b2 && !up && !down && (pdd4 & 0x02)) {
		up = HK_UP;
		b2 = HK_B2;
	}

	sync = !(pdd4 & 0x10) ? HK_SYNC : 0;

	return pwr | sync | up | down | b1 | b2 | b3 | b4;
}

unsigned char keys_wait() {
	short i;
	key = 0x00;
ponovo:
	while (!key) HALT();
	for (i = 0; key; ++i) {
		if (key & 0x01)
			return 1 << i;
		key >>= 1;
	}
	goto ponovo;	// make compiler happy... :)
}

static void keys_irq() {
	PDIRQEG = 0x00;
	key = keys_read();
	PDIRQEG = 0x0F;
}

void keys_init() {
	// hardkeys
	KEYMASK(0);
	PDPOL = 0x0F;
	PDIRQEG = 0x0F;
	PDKBEN = 0x00;
	PDIRQEN = 0x0F;
	PDDATA |= 0x0F;

	// hotsync
	ICR &= ~0x8000;
	ICR |= 0x0800;
	ISR |= INTR_IRQ1;

	IMR &= ~(INTR_INT0 | INTR_INT1 | INTR_INT2 | INTR_INT3 | INTR_IRQ1);

	old_keys_irq = irq_register(IRQID_KEYS, keys_irq);
}

void keys_deinit() {
	// TODO dovrsiti
	PDIRQEN = 0x00;
	PDIRQEG = 0x00;
	IMR |= INTR_INT0 | INTR_INT1 | INTR_INT2 | INTR_INT3 | INTR_IRQ1;
	irq_register(IRQID_KEYS, old_keys_irq);
}
