#include "../include/cpu.h"

#include "lcd.h"
#include "string.h"
#include "font.h"

#define BIAS_D	0x01
#define BIAS_S	0x04
#define BIAS_C	0x80

#define PULSE(port, bit) {port |= (bit); port &= ~(bit);}
#define BIAS_CLOCK() PULSE(PFDATA, BIAS_C)
#define BIAS_STROBE() PULSE(PFDATA, BIAS_S)

short LCD_x, LCD_y;

void LCD_bias(short val) {
	short b;
//	PFDIR |= BIAS_D;
	PFDATA &= ~BIAS_C;
	for (b = 0; b < 8; ++b) {
		PFDATA = (PFDATA & ~1) | !!(val & 0x80);
		val <<= 1;
		BIAS_CLOCK();
	}
	BIAS_STROBE();
	PFDATA |= BIAS_C;
//	PFDIR &= ~BIAS_D;
}

/*
	PB.0 => kbd row 0
	PB.1 => LCD vcc
	PB.3 => kbd row 1
	PB.6 => kbd row 2

	PD.0 =< key col 0 (INT0)
	PD.1 =< key col 1 (INT1)
	PD.2 =< key col 2 (INT2)
	PD.3 =< key col 3 (INT3)
	PD.4 =< dock button (IRQ1)
	PD.5 => serial enable
	PD.6 => LCD bias on
	PD.7 =< power fail (IRQ6)

	PF.0 => LCD bias data
	PF.1 =< pen interrupt (IRQ5)
	PF.2 => LCD bias strobe
	PF.5 => LCD backlight
	PF.6 => LCD display on
	PF.7 => LCD bias clock

	PG.0 =< DTACK
	PG.2 =< ID select (drives kbd) ???
	PG.3 => enable A/D voltage divider
	PG.4 => enable irda
	PG.5 => A/D select
*/

static void LCD_updatecursor() {
	LCXP = 0x8000 | (LCD_x << 3);
	LCYP = LCD_y << 3;
}

void LCD_clear() {
	memsetw(display, 0, 160*160/16);

	LCD_x = 0;
	LCD_y = 0;

	LCD_updatecursor();
}

void LCD_init() {
	short i;

	// controler
	LSSA = (unsigned long)display;
	LVPW = 0x0A;
	LXMAX = 0xA0;
	LYMAX = 0x9F;
	LPICF = 0x08;
	LPOLCF = 0x00;
	LACDRC = 0x00;
	LPXCD = 0x00;
	LCKCON = 0x86;
	LRRA = 0xFE;
	LPOSR = 0x0;
	LFRCM = 0xB9;
	LGPMR = 0xA5;
	PWMR = 0x018A;

	// cursor
	LBLKC = 0xBF;
	LCWCH = 0x0808;

	PBDATA |= 0x02;	// lcd on

	for (i = 0; i < 32; ++i) {
		while (PLLFSR & 0x8000);
		while (!(PLLFSR & 0x8000));
	}

	PDDATA |= 0x40;	// lcd bias on
	LCD_bias(0x80);

	LCD_clear();
}

void LCD_scroll() {
	memmovew(display, (char*)display+160*8/8, (160*160-160*8)/16);
	memsetw((char*)display+(160*160-160*8)/8, 0, 160*8/16);
}

static void LCD_newline() {
	LCD_x = 0;

	if (LCD_y < 19)
		LCD_y++;
	else
		LCD_scroll();

	LCD_updatecursor();
}

void LCD_putc(char c) {
	switch (c) {
		case '\n':
			LCD_newline();
			return;
		case '\r':
			LCD_x = 0;
			LCD_updatecursor();
			return;
		case '\b':
			if (LCD_x) {
				--LCD_x;
				LCD_updatecursor();
			}
			return;
	}

	if (c >= 0) {
		short i;
		for (i = 0; i < 8; ++i)
			display[(LCD_y << 3) + i][LCD_x] =
				font_mono[(unsigned char)c][i];
	}

	if (++LCD_x >= 20)
		LCD_newline();
	LCD_updatecursor();
}
