#include <elf/elf.h>
#include <string.h>

static const char elf_magic[] = {ELFMAG0, ELFMAG1, ELFMAG2, ELFMAG3};

/*!
 * Provjeri da li se uopce radi o elf fajli.
 * \param elf Elf
 * \returns 0 ako nije
 */
char is_elf(const Elf *elf) {
	return !memcmp(&elf->header.e_ident[EI_MAG0], elf_magic, sizeof(elf_magic));
}

/*!
 * Provjeri da li je elf fajla misljena za pokretanje.
 * \param elf Elf
 * \returns 0 ako nije
 */
short int elf_check_executable(const Elf *elf) {
	const Elf32_Ehdr *hdr = &elf->header;

	// da li je stvarno elf?
	if (!is_elf(elf))
		return ELFE_NOT_ELF;

#if ARCH_BITS == 32
	// da li je 32 bitan?
	if (hdr->e_ident[EI_CLASS] != ELFCLASS32)
		return ELFE_NOT_32BIT;
#else
#	error Dodati provjeru za tu arhitekturu
#endif


#if ARCH_LITTLE_ENDIAN
	// da li je little endian?
	if (hdr->e_ident[EI_DATA] != ELFDATA2LSB)
		return ELFE_NOT_LITTLE_ENDIAN;
#elif defined(ARCH_BIG_ENDIAN)
	// da li je big endian?
	if (hdr->e_ident[EI_DATA] != ELFDATA2MSB)
		return ELFE_NOT_BIG_ENDIAN;
#else
# error Endianess undefined!
#endif

	// verzija?
	if (hdr->e_ident[EI_VERSION] != hdr->e_version ||
	    hdr->e_ident[EI_VERSION] < EV_1)
		return ELFE_INVALID_VERSION;

	// egzekjutibl?
	if ((hdr->e_type != ET_EXEC && hdr->e_type != ET_DYN) || !hdr->e_entry)
		return ELFE_NOT_EXE;

	// m68k?
	if (hdr->e_machine != EM_68K)
		return ELFE_NOT_68K;

	// da slucajno ne fali program header
	if (!hdr->e_phoff)
		return ELFE_NO_PROGRAM_HEADER;

	// da li je heder dobre velicine?
	if (hdr->e_ehsize < sizeof(*hdr))
		return ELFE_CRIPPLED_HEADER;	

	return ELFE_OK;
}

const void * elf_get_entry_ra(const Elf *prog) {
	return (const char*)prog + (unsigned int)prog->header.e_entry;
}
