#include <elf/elf.h>
#include <string.h>

short int elf_get_num_phdrs(const Elf *prog) {
	return (short int)prog->header.e_phnum;
}

const Elf32_Phdr* elf_get_phdr(const Elf *prog, short int id) {
	if (id >= elf_get_num_phdrs(prog))
		return (Elf32_Phdr*)0;

	return (const Elf32_Phdr*)(
			(const char*)prog +
			prog->header.e_phoff +
			prog->header.e_phentsize * id
		);
}

const Elf32_Phdr* elf_find_phdr_by_va(const Elf *prog, const void *va) {
	short int i;

	for (i = 0; i < elf_get_num_phdrs(prog); ++i) {
		const Elf32_Phdr *ph = elf_get_phdr(prog, i);
		if (ph->p_type != PT_LOAD)
			continue;
		if (va >= ph->p_vaddr && va < (void*)((char*)ph->p_vaddr + ph->p_memsz))
			return ph;
	}
	return (Elf32_Phdr*)0;
}
