#include <elf/elf.h>
#include <string.h>

short int elf_get_num_shdrs(const Elf *prog) {
	return (short int)prog->header.e_shnum;
}

const Elf32_Shdr* elf_get_shdr(const Elf *prog, short int id) {
	if (id >= elf_get_num_shdrs(prog))
		return (Elf32_Shdr*)0;

	return (const Elf32_Shdr*)(
			(const char*)prog +
			prog->header.e_shoff +
			prog->header.e_shentsize * id
		);
}

const char * elf_get_string(const Elf *prog, const Elf32_Shdr *sh, short int id) {
	if (!sh || sh->sh_type != SHT_STRTAB)
		return NULL;

	return (const char *)prog + sh->sh_offset + id;
}

const Elf32_Shdr * elf_find_shdr(const Elf *prog, const char *name) {
	const Elf32_Shdr *sh, *strs = elf_get_shdr(prog, prog->header.e_shstrndx);
	short int i;

	sh = (const Elf32_Shdr*)((const char*)prog + prog->header.e_shoff);
	for (i = 0; i < elf_get_num_shdrs(prog); ++i) {
		const char *sname = elf_get_string(prog, strs, sh->sh_name);
		if (sname && !strcmp(sname, name))
			return sh;
		sh = (const Elf32_Shdr*)((const char*)sh + prog->header.e_shentsize);
	}
	return (Elf32_Shdr*)0;
}
