#include <flash/flash.h>
#include <types.h>

static char waitflashop(unsigned long addr, unsigned char qn) RAMTEXT;
static char writeword(unsigned long addr, unsigned short d) RAMTEXT;

unsigned short flash_getid() {
	unsigned short id;
	FLASH_RESET();
	FLASH_CMD(0x90);
	id = FLASH[1];
	FLASH_RESET();
	return id;
}

static char waitflashop(unsigned long addr, unsigned char qn) {
	unsigned short l;
	while (((l = FLASH[addr]) ^ FLASH[addr]) & qn) {
		if (l & 0x20)
			return FLASH_E_TIMEOUT;
	}
	return FLASH_E_OK;
}

static char writeword(unsigned long addr, unsigned short d) {
	FLASH[addr] = 0xA0;
	FLASH[addr] = d;
	return waitflashop(addr, 0x40);
}

char flash_write(unsigned long addr, unsigned short *data, size_t nwords) {
	char err = FLASH_E_OK;

	if (addr & 1) return FLASH_E_ODD_ADDR;

	FLASH_RESET();
	FLASH_CMD(0x20);
	for (addr >>= 1; nwords; --nwords, ++addr, ++data)
		if ((err = writeword(addr, *data)))
			break;
	FLASH[0] = 0x90;
	FLASH[0] = 0xF0;
	return err;
}

unsigned long flash_address_by_id(unsigned char id) {
	static const unsigned long bsa[] = {0x00000, 0x04000, 0x06000, 0x08000};

	if (id > 34) return -1UL;
	if (id < 4) return bsa[id];
	return (id - 3) << 16;
}

unsigned short flash_get_num_sectors() {
	return 34;	// nije baš neko rješenje... D:
}

char flash_erase_sector(unsigned char id) {
	unsigned long saddr = flash_address_by_id(id) >> 1;
	FLASH_RESET();
	FLASH_CMD(0x80);
	FLASH[0x555] = 0xAA;
	FLASH[0x2AA] = 0x55;
	FLASH[saddr] = 0x30;
	return waitflashop(saddr, 0x04);
}

char flash_erase() {
	FLASH_RESET();
	FLASH_CMD(0x80);
	FLASH[0x555] = 0xAA;
	FLASH[0x2AA] = 0x55;
	FLASH[0x555] = 0x10;
	return waitflashop(0, 0x04);
}
